@@ -6,11 +6,14 @@ Apache2::Controller - framework for Apache2 handler apps
 
 =head1 VERSION
 
-Version 0.110.000 - BETA TESTING (ALPHA?)
+Version 0.0.2
 
 =cut
 
-our $VERSION = version->new('0.110.000');
+use strict;
+use warnings FATAL => 'all', NONFATAL => 'redefine';
+
+our $VERSION = '0.0002';
 
 =head1 SYNOPSIS
 
@@ -26,7 +29,8 @@ method for the uri.
  use base qw( Apache2::Controller );
 
  use Apache2::Const -compile => qw( :http );
- sub allowed_methods {qw( default bar baz )}
+ use Readonly;
+ Readonly our @ALLOWED_METHODS => qw( default bar baz );
 
  # suppose '/foo' is the uri path dispatched to this controller
  # and your dispatch uses L<Apache2::Controller::Dispatch::Simple>
@@ -41,14 +45,13 @@ method for the uri.
 
  # http://myapp.xyz/foo/bar/biz/schnozz
  sub bar {
-     my ($self, @path_args) = @_;             
-     # @path_args is:
-     #      qw( biz schnozz )
-     #      @{ $self->{path_args} }
-     #      @{ $self->pnotes->{path_args} }
-
+     my ($self) = @_;
+     my @path_args = @{ $self->{path_args} }; # qw( biz schnozz )
      $self->content_type('text/html');
-     $self->print(q{ <p>"WE ARE ALL KOSH"</p> });
+     $self->print(q{
+        <p>Sometimes you eat the bear, 
+        and sometimes the bear eats you.</p>
+     });
      return Apache2::Const::HTTP_OK;
  }
 
@@ -68,11 +71,11 @@ method for the uri.
      return Apache2::Const::HTTP_OK;
  }
 
- 1;
+1;
 
 You could implement a pretty nice REST interface, or any other kind
 of HTTP-based API, by returning the appropriate HTTP status codes.
-See L<Apache2::Controller::Refcard/status> for a list.
+See L<Apache2::Controller::Refcard/http_status> for a list.
 
 See L<Apache2::Controller::Render::Template> for an additional base
 for your controller class to render HTML with L<Template> Toolkit.
@@ -82,13 +85,12 @@ for your controller class to render HTML with L<Template> Toolkit.
 Apache2::Controller is a lightweight controller framework for 
 object-oriented applications designed to run only under mod_perl 
 children in high-performance Apache2 handler modules.  It features URL 
-dispatch with flexible configuration, auth plugins, a cookie tracker
-for Apache::Session, liberty for any storage models that work under mod_perl,
-rendering using Template Toolkit or direct printing with Apache,
-and base inheritance configuration allowing you to 
+dispatch with flexible configuration, auth plugins for OpenID, a cookie 
+session and MySQL store, a pluggable infrastructure for your own models 
+and views, and base inheritance configuration allowing you to 
 construct your applications as you need, without trying to be all things 
-to all people or assimilate the world.  
-It is intended as a framework for 
+to all people or assimilate the world, and without having to load all 
+site modules for every page handler. It is intended as a framework for 
 new applications specialized as Apache2 handlers, not as a means to 
 absorb existing applications or to create portable code.  
 
@@ -144,67 +146,35 @@ Apache2 server configuration.  Instead you make a subclass
 of L<Apache2::Controller::Dispatch> and use that as a
 PerlInitHandler.  It will map a URI to an appropriate
 Apache2::Controller subclass object and method and will
-use C<<$r->push_handlers()>> if successful to push Apache2::Controller
+use $r->push_handlers() if successful to push Apache2::Controller
 onto the modperl handler stack.  See L<Apache2::Controller::Dispatch>
 for more information and different types of URI dispatching.
 
 =head1 OTHER REQUEST PHASE HANDLERS
 
-Configure other handlers in your config file to set things up
-before your Apache2::Controller runs.
+Add handlers in your config file your own modules which 
+C<use base> to inherit from these classes if you need
+to add them to your config file:
 
-Most of these handlers use L<Apache2::Controller::NonResponseBase>
-as a base for the object, which usually does not need to
-instantiate the L<Apache2::Request> object.
-So if your subclass methods need acces to the
-L<Apache2::RequestRec> object C<<$r>>, 
-it is in C<<$self->{r}>>.
+=over 4
 
-Some other request phase handlers register later-stage handlers,
-for example to save the session with a C<PerlCleanupHandler>
-after the controller successfully completes the C<Response> phase.
+=item * PerlInitHandler L<Apache2::Controller::Session::Cookie> 
 
-Handlers before the response stage will return DECLINED or DONE if 
-necessary to prevent running your Apache2::Controller in the
-case of an error.  If so, no further handlers will be run,
-such as for the PerlCleanupHandler stage.
+Provides a cookie-based 
+session mechanism using YAML::Syck as the serializer, and requires DBI 
+connection information.  If you want to use some other session 
+mechanism, you can plug it in as a different module.
 
-The controller handler always returns your set HTTP status code
-or OK (0) to Apache.  (Does this mean PerlCleanupHandlers will run?)
+=item * PerlAuthenHandler L<Apache2::Controller::Auth::OpenID> subclass
 
-Add handlers in your config file with your own modules which 
-C<use base> to inherit from these classes as you need them:
+Implements OpenID logins and redirects to your specified login page.
 
-=head2 PerlHeaderParserHandler Apache2::Controller::Session
+=item * PerlAuthzHandler L<Apache2::Controller::Groups> subclass
 
-C<<$r->pnotes->{session}>> fed from and stored to an 
-L<Apache::Session> tied hash.  Pushes a PerlCleanupHandler
-to save the session after the main controller returns OK.
+=back
 
-See L<Apache2::Controller::Session>
-and L<Apache2::Controller::Session::Cookie>.
-
-=head2 PerlAuthenHandler Apache2::Controller::Authen::OpenID
-
-Implements OpenID logins and redirects to your specified login 
-controller by changing the dispatch selection on the fly.
-When authorized, username is put in C<<$r->pnotes->{session}{username}>>
-with a timestamp, and denies access after a set idle timeout.
-Requires use of Apache2::Controller::Session.
-
-UNIMPLEMENTED
-
-See L<Apache2::Controller::Authen::OpenID>.
-
-=head2 PerlAuthzHandler Apache2::Controller::Authz::Groups subclass
-
-Subclass this handler to implement groups, and configure
-by implementing routines that load config files or query 
-a database.
-
-UNIMPLEMENTED
-
-See L<Apache2::Controller::Authz::Groups>.
+These handlers for other stages will return DECLINED or DONE if 
+necessary to prevent running your Apache2::Controller.
 
 =head1 Apache2::Controller RESPONSE PHASE HANDLER
 
@@ -212,16 +182,16 @@ Apache2::Controller is pushed as the PerlResponseHandler if
 the dispatch class finds a valid module and method for the request.
 Your controller uses
 
-=head2 SUBCLASS OF L<Apache2::Request>
+=head1 SUBCLASS OF L<Apache2::Request>
 
 Apache2::Controller is a subclass of L<Apache2::Request>,
 which inherits the L<Apache2::RequestRec> object with most of
 the modperl2 request extension libraries loaded during 
-construction.  So you can call C<<$self->$methodname>> for any of
+construction.  So you can call $self->$methodname for any of
 the methods associated with L<Apache2::Request>, 
 L<Apache2::RequestRec> and some of their friends.  
 Watch the log for warnings about redefined subroutines, or 
-C<<use warnings FATAL => 'all'>> to keep yourself on the
+C<use warnings FATAL =E<gt> 'all'> to keep yourself on the
 right track.
 
  package MyApp::C::SomeURIController;
@@ -230,7 +200,7 @@ right track.
  sub set_shipping_address {
     my ($self) = @_;
 
-    # $self->param() is Apache::Request param():
+    # $self->param() automatically calls Apache::Request param():
     my ($shipto, $addr, $zip) 
         = map {
             my $detaint = "detaint_$_";
@@ -240,8 +210,6 @@ right track.
  }
 
  sub detaint_shipto {   # ...
- sub detaint_addr   {   # ...
- sub detaint_zip    {   # ...
 
 At any rate, your Apache2::Controller child object subclasses 
 itself into Apache2::Request and delegates all those methods
@@ -249,7 +217,7 @@ to the internal hash value $self->{r}, which is the actual
 Apache2::Request object.
 See L<Apache2::Request> about those gory details.  Whether 
 you call C<< $self->$apache2_request_method >> or 
-C<< $self->{r}->$apache2_request_method >> matters not,
+C<< $self->{r}-E<gt>$apache2_request_method >> matters not,
 you still ask the same object, so you might as well use
 C<< $self->... >> to make it look clean.
 
@@ -260,16 +228,13 @@ act accordingly, set the right things for C<Apache2::RequestRec>
 and return the right C<Apache2::Const/:http>.
 
 In the event of an error, if you wish, use L<Apache2::Controller::X>
-and throw one with field 'status' set to a valid HTTP return code.  
-This lets you implement nice error templates if your controller uses 
-L<Apache2::Controller::Render::Template> as a second base.
+and throw with field 'http_status' set to a valid HTTP return code.  
 See L<ERRORS> below.
 
 Success in the controller method normally should just return the
-appropriate HTTP status code.  You can return HTTP_OK (200) if that
-is what you mean, or it is the default status if you return OK (0).
+appropriate HTTP status code.
 
-Or, if you do C<< $self->status( Apache2::Const::HTTP_SOMETHING ) >>
+Or, if you do C<< $self->http_status( Apache2::Const::HTTP_SOMETHING ) >>
 and then just return, Apache2::Controller will not override the set status.
 
 See L<Apache2::Controller::Refcard> for a list of HTTP return constants
@@ -281,7 +246,7 @@ L<Apache2::Controller::X> is the L<Exception::Class> hierarchy used
 to kick up errors and redirects.  You can use or subclass this,
 or you can throw your own exceptions with L<Exception::Class> 
 or another exception object, or just C<die()>, but you either 
-die with an Apache2::Controller::X subclass with an status 
+die with an Apache2::Controller::X subclass with an http_status 
 field or should not die and return an HTTP status code instead.
 
 See L<Apache2::Controller::X> for help on throwing exceptions
@@ -300,7 +265,7 @@ or exception as the first argument.
  # ...
  sub error {
      my ($self, $X) = @_;
-     $self->status( Apache2::Const::HTTP_BAD_REQUEST );
+     $self->http_status( Apache2::Const::HTTP_BAD_REQUEST );
      $self->content_type('text/plain');
      $self->print("Take this job and shove it!\n", "\n", $X, "\n");
      if ($X->isa('Apache2::Controller::X')) {
@@ -312,7 +277,7 @@ or exception as the first argument.
 
 Of course, all exceptions are sent to the error log using
 L<Log::Log4perl> DEBUG() before the handler completes, and
-any refusal status greater or equal to 400 (HTTP_BAD_REQUEST) 
+any refusal http_status greater or equal to 400 (HTTP_BAD_REQUEST) 
 will be written to the access log with L<Apache2::Log> log_reason() 
 using the first few characters of the error.
 
@@ -336,7 +301,7 @@ Devel::Size to keep memory usage down.
 Your controller should set content type with C<$self-E<gt>content_type($ct)>
 to something specific if you need that.  Otherwise it will let
 mod_perl set it to whatever it chooses when you start to print.
-This is usually text/html.
+I guess this is usually text/html.
 
 =head1 REDIRECTS
 
@@ -346,12 +311,8 @@ object which isa Apache2::Controller::X::Redirect.
 
  Apache2::Controller::X::Redirect->throw("http://perl.apache.org");
 
-Apache2::Controller::handler() will trap the exception
-and send redirects on their way.  See L<Apache2::Controller::X>.
-
-If you don't want to terminate the response chain, maybe you can set
-C<<$self->status(REDIRECT)>> and C<<$self->headers_out->{Location}>>...
-so the session would still be saved, for example.  Experiment.
+Apache2::Controller::handler() will trap the exception and send
+redirects on their way.  See L<Apache2::Controller::X>.
 
 =head1 LOGGING
 
@@ -387,38 +348,24 @@ child interpreter.  So,
 you can use package namespace effectively to store data
 that will persist in the mod_perl child across requests.
 
-=head1 LOAD BALANCING
+=head1 WARNINGS AND CAVEATS
 
-A2C does not have to load 
-all site modules for every page handler, which could help with load-balancing
-highly optimized handlers for specific URI's while having a universal
-application installer.  
+=head2 Handle errors appropriately
 
-Picture if you will, a programming utopia in which all engineers
-are respected, highly paid and content.  
+If you return an error code of 400 HTTP_BAD_REQUEST or higher,
+Apache2::Controller will return DONE instead of OK, so Apache2
+will not process any further handlers in the stack but will
+end the request right here.
 
-You deploy the same Apache, the
-same CPAN modules and your whole application package to every server,
-and attach a url-generating subroutine to the L<Template|Template> stash
-that puts in a different hostname when the URI is one of your
-load-intensive functions.  
-
- <a href="[% myurl('/easy') %]">easy</a> "http://pony.x.y/easy"
-
- <a href="[% myurl('/hard') %]">hard</a> "http://packhorse.x.y/hard"
-
-Web designers can be taught to use this function C<< myurl() >>, 
-but system admins
-maintain the map that it loads to figure out what servers to use.
-
-Then the Apache2 config files on those
-packhorse servers would pre-load only the subclassed controllers
-that you needed, and redirect all other uri requests to the pony servers.
+So, it's up to you in your controllers to use C<eval { }> appropriately
+and deal with errors, for instance, if your database transaction
+goes awry, it's up to you to roll back a transaction in the 
+controller module.  Don't push a PerlCleanupHandler to do that,
+though it's tempting to do so, because if you return an error code
+from the controller method, the PerlCleanupHandler will not run.
 
 =cut
 
-use strict;
-use warnings FATAL => 'all';
 use English '-no_match_vars';
 
 use base qw( Apache2::Request Apache2::Controller::Methods );
@@ -448,9 +395,163 @@ use Apache2::Const -compile => qw( :common :http );
 
 =head1 FUNCTIONS
 
-=head2 new
+=head2 Apache2::Controller::handler( Apache2::RequestRec object )
 
- $handler = MyApp::C::ControllerSubclass->new( Apache2::RequestRec object )
+The handler is pushed from an Apache2::Controller::Dispatch
+subclass and should not be set in the config file.  It looks
+for the controller module name in C<< $r->notes->{controller} >>
+and for the method name in C<< $r->notes->{method} >>.
+
+It runs start_request() if implemented, then runs the
+selected method, then runs end_request() if implemented,
+setting the http_status to any positive return value
+returned by any of those methods.  (The last positive
+value returned will be the final http_status.)
+
+Errors are intercepted and if the handler object was created
+and implements an C<< $handler->error($exception) >> method 
+then the exception will be passed as the argument.
+
+An HTTP status code of HTTP_BAD_REQUEST or greater will 
+cause log_reason to be called with a truncated error string
+and the uri for recording in the access log.
+
+=head1 RUN_ALL mode
+
+In the odd chance you plan to push further PerlResponseHandlers
+from your controller or server config, maybe to print
+debugging data at the end in HTML comments for instance, 
+then you need to set a boolean directive (temporarily a C<PerlSetVar>
+variable)
+called C<A2CRunAllResponseHandlers>. Then C<Apache2::Controller>
+will check Apache's stack of PerlResponseHandlers for the Response
+phase of the request lifecycle, to see whether you have pushed
+more response handlers to the stack, and if so will run them.
+Otherwise, the Response phase will stay in default RUN_FIRST 
+mode and no pushed PerlResponseHandlers will be executed.
+
+=cut
+
+my %supports_error_method = ( );
+
+sub handler : method {
+    my ($class, $r) = @_;
+    return $class if !defined $r;
+
+    my $method = $r->notes->{method};
+
+    DEBUG("$class -> $method");
+
+    my ($handler, $run_all, $http_status, $X) = ( );
+    eval {
+
+        $handler = $class->new($r);
+        $run_all = $handler->get_directive('A2CRunAllResponseHandlers');
+        $method  = $handler->{method};
+
+        DEBUG("executing $class -> $method()");
+        $http_status = $handler->$method();
+      # $r->pnotes->{session} = $handler->{session} if exists $handler->{session};
+        $http_status = $handler->status() if !defined $http_status;
+    };
+    if ($X = $EVAL_ERROR) {
+        my $ref = ref($X);
+
+        if (blessed($X) && $X->can('http_status')) {
+            $http_status = $X->http_status;
+        }
+        $http_status = Apache2::Const::SERVER_ERROR if !defined $http_status;
+
+        $http_status = Apache2::Const::HTTP_OK 
+            if $http_status == Apache2::Const::OK;
+
+        # if a redirect, just set the location and status
+        if ($ref && $X->isa('Apache2::Controller::X::Redirect')) {
+            $http_status = Apache2::Const::REDIRECT;
+            $r->err_headers_out->add(Location => "$X");
+        }
+
+        # else process the error
+        else {
+
+            # if appropriate and able to call self->error(), do that now
+            if ($handler && !$r->notes->{use_standard_errors}) {
+
+                eval {
+                    if (exists $supports_error_method{$class}) {
+                        $handler->error($X);
+                    } 
+                    elsif ($class->can('error')) {
+                        $handler->error($X); 
+                    }
+                };
+                if (my $tempX = Exception::Class->caught('Apache2::Controller::X')) {
+                    $X = $tempX;
+                }
+                elsif ($EVAL_ERROR) {
+                    $X = "$EVAL_ERROR";
+                }
+            }
+
+            # now decide how to output debugging log:
+            if (ref($X) && $X->isa('Apache2::Controller::X')) {
+                DEBUG(sub {
+                    ref($X).": $X\n".($X->dump ? Dump($X->dump) : '').$X->trace()
+                });
+            }
+            else {
+                DEBUG("Caught an unknown error: $X");
+            }
+            INFO("Error caught in Apache2::Controller::handler: $X\n");
+        }
+    }
+
+    DEBUG("done with handler processing...");
+    DEBUG(sub { "content_type: ".$r->content_type() });
+    
+    $http_status ||= Apache2::Const::HTTP_OK;
+    $r->status($http_status);
+
+    DEBUG(sub { 
+        "http_status: $http_status ".status_message($http_status)
+        .", status_line for ".$r->status." is '".$r->status_line()."'"
+    });
+
+    # if http_status is an erorr, file error (possibly truncated) as a 
+    # log_reason in the access log for why this request was denied.
+    if ($http_status >= Apache2::Const::HTTP_BAD_REQUEST) {
+        log_bad_request_reason($r, $X);
+        $r->status_line("$http_status ".status_message($http_status)) 
+            if !$r->status_line;
+        return Apache2::Const::OK;
+    }
+    elsif ($run_all) {
+        # return OK or DECLINED depending on whether the controller module
+        # or apache config pushed any further handlers.  essentially switch
+        # further PerlResponseHandlers from RUN_FIRST to RUN_ALL mode.
+        my @resp_handlers = map { eval { $_->() } } 
+            @{ $r->get_handlers('PerlResponseHandler') || [ ] };
+
+        my $last_resp_h = $resp_handlers[-1];
+
+        return defined $last_resp_h && $last_resp_h eq $class
+            ? Apache2::Const::OK
+            : Apache2::Const::DECLINED;
+        
+        return Apache2::Const::OK;
+    }
+    else {
+        return Apache2::Const::OK;
+    }
+
+    # supposedly you can return the http status, but it doesn't work right
+    # if you return HTTP_OK.  shouldn't it?
+    # i prefer to directly control what is happening
+    # by setting the status and then telling Apache whether
+    # to continue or to stop processing the request if not.
+}
+
+=head2 MyApp::C::ControllerSubclass->new( Apache2::RequestRec object )
 
 This is called by handler() to create the Apache2::Controller object
 via the module chosen by your L<Apache2::Controller::Dispatch> subclass.
@@ -472,28 +573,12 @@ starts, you can override the constructor.
          my ($r) = @_;
          my $dbh = $r->pnotes->{dbh};
          $dbh->rollback() unless $r->notes->{commit_success};
-     });    # implements some convention that you set 'commit_success', say
+     });
      return $self;
  }
 
-Similarly, to do something always at the end of every 
-request, from within the dispatched PerlResponseHandler:
-
- package MyApp::DestroyController;
- use Devel::Size;
- use Log::Log4perl qw(:easy);
- my $MAX = 40 * 1024 * 1024;
- sub DESTROY {
-     my ($self) = @_;
-     my $size = total_size($self);  # whoo baby!
-     INFO("size of $self->{class} is bigger than $MAX!") if $size > $MAX;
-     return; # self is destroyed
- }
-
-And your subclass is:
-
- package MyApp::Controller::Foo;
- use base qw( Apache2::Controller MyApp::InitController MyApp::DestroyController );
+ package MyApp::C::Foo;
+ use base qw( Apache2::Controller MyApp::InitController );
 
  # ...
 
@@ -543,8 +628,6 @@ sub new {
     if ($session) {
         $self->{session} = $session;
         DEBUG(sub{"found and attached session to controller self:\n".Dump($session)});
-        # this is the same reference as the pnotes reference still,
-        # so the cleanup handler will find all changes made to it
     }
 
     DEBUG(sub { Dump({
@@ -556,143 +639,58 @@ sub new {
 
 =head1 METHODS
 
-Methods are also extended by 
-L<Apache2::Controller::Methods|Apache2::Controller::Methods>.
+=head2 $self->fields( @list_of_field_names )
 
-=head2 handler
-
- # called from Apache, your subclass pushed to PerlResponseHandler
- # by your A2C dispatch handler:
- MyApp::Controller::Foo->handler( Apache2::RequestRec object )
-
-The handler is pushed from an Apache2::Controller::Dispatch
-subclass and via your dispatched subclass of Apache2::Controller.
-It should not be set in the config file.  It looks
-for the controller module name in C<< $r->notes->{controller} >>
-and for the method name in C<< $r->notes->{method} >>.
-
-Errors are intercepted and if the handler object was created
-and implements an C<< $handler->error($exception) >> method 
-then the exception will be passed as the argument.
-
-An HTTP status code of HTTP_BAD_REQUEST or greater will 
-cause log_reason to be called with a truncated error string
-and the uri for recording in the access log.
+Returns hash reference of query params from a list of field names.
+Uses the common convention of returning more than one value if
+present in the query, it 
+returns an array ref, or it returns a scalar if only one value.
+If no field names passed, returns a hash ref of all query params.
 
 =cut
 
-my %supports_error_method = ( );
+sub fields {
+    my ($self, @keys) = @_;
+    @keys = $self->param if !@keys;
+    my $fields = { };
 
-sub handler : method {
-    my ($class, $r) = @_;
-    return $class if !defined $r;
-
-    my $method = $r->notes->{method};
-
-    DEBUG("$class -> $method");
-
-    my ($handler, $status, $X) = ( );
-    eval {
-
-        $handler = $class->new($r);
-        $method  = $handler->{method};
-
-        DEBUG("executing $class -> $method()");
-        my $args = $r->pnotes->{path_args} || [];
-        $status = $handler->$method(@{$args});
-        $status = $handler->status() if !defined $status;
-
-        Apache2::Controller::X->throw("must set http status >= 0") 
-            if defined $status && $status < 0;
-    };
-    if ($X = $EVAL_ERROR) {
-        my $ref = ref($X);
-
-        if (blessed($X) && $X->can('status')) {
-            $status = $X->status;
-        }
-        $status = Apache2::Const::SERVER_ERROR if !defined $status;
-
-        $status = Apache2::Const::HTTP_OK 
-            if $status == Apache2::Const::OK;
-
-        # if a redirect, just set the location and status
-        if ($ref && $X->isa('Apache2::Controller::X::Redirect')) {
-            $status = Apache2::Const::REDIRECT;
-            $r->err_headers_out->add(Location => "$X");
-        }
-
-        # else process the error
-        else {
-
-            # if appropriate and able to call self->error(), do that now
-            if ($handler && !$r->notes->{use_standard_errors}) {
-
-                eval {
-                    if (exists $supports_error_method{$class}) {
-                        $handler->error($X);
-                    } 
-                    elsif ($class->can('error')) {
-                        $handler->error($X); 
-                    }
-                };
-                if (my $tempX = Exception::Class->caught('Apache2::Controller::X')) {
-                    $X = $tempX;
-                }
-                elsif ($EVAL_ERROR) {
-                    $X = "$EVAL_ERROR";
-                }
-            }
-
-            # now decide how to output debugging log:
-            if (ref($X) && $X->isa('Apache2::Controller::X')) {
-                DEBUG(sub {
-                    ref($X).": $X\n".($X->dump ? Dump($X->dump) : '').$X->trace()
-                });
-            }
-            else {
-                DEBUG("Caught an unknown error: $X");
-            }
-            INFO("Error caught in Apache2::Controller::handler: $X\n");
-        }
+    for my $key (@keys) {
+        my @values = $self->param($key);
+        $fields->{$key} = @values > 1 ? \@values : $values[0];
     }
 
-    DEBUG("done with handler processing...");
-    DEBUG(sub {
-        my $ctype = $r->content_type();
-        "content type is ".($ctype || '[undef]');
-    });
-    
-    $r->status($status) if $status;
+    return $fields;
+}
 
-    DEBUG(sub { 
-        my $stat  = defined $status ? $status : '';
-        my $sline = $r->status_line() || '[none]';
-        "status: $status "
-        .($status ? status_message($status) : 'N/A')
-        .", status_line is '$sline'";
-    });
+=head2 $self->server_url( )
 
-    if (!defined $status) {
-        return Apache2::Const::OK;
-    }
-    elsif ($status >= Apache2::Const::HTTP_BAD_REQUEST) {
-        # if status is an error, file error (possibly truncated) as a 
-        # log_reason in the access log for why this request was denied.
-        # (is this desirable?)
-        log_bad_request_reason($r, $X);
-        $r->status_line("$status ".status_message($status)) 
-            if !$r->status_line;
-        return $status;
-    }
-    else {
-        return $status == Apache2::Const::HTTP_OK
-            ? Apache2::Const::OK
-            : $status;
-    }
+This returns 'protocol://' concatenated with $r->get_server_name,
+and if $r->get_server_port is anything other than the protocol's
+standard port number, it will concat ':portnum'.
 
-    # supposedly you can return the http status, but it doesn't work right
-    # if you return HTTP_OK.  shouldn't it?
+=cut
+
+sub server_url {
+    my ($self) = @_;
+    Apache2::Controller::X->throw('unimplemented.  complicated!');
+    return 'http://'.$self->get_server_name();
+}
+
+=head2 $self->root_directory( )
+
+$r->document_root with everything after last / cut out.  Why?
+What use is this?
+
+=cut
+
+my %root_dirs = ( );
+sub root_directory {
+    my ($self) = @_;
+    my $class = $self->{class};
+    if (!exists $root_dirs{$class}) {
+        ($root_dirs{$class} = $self->document_root()) =~ s{ / [^/]* \z }{}mxs;
+    }
+    return $root_dirs{$class}; 
 }
 
 =head1 USING INHERITANCE
@@ -716,7 +714,7 @@ they should add a line to call a shared cleanup method.
 
  package MyApp::C::Foo;
  use base qw( Apache2::Controller MyApp::Cleanup );
- sub allowed_methods {qw( foo bar )}
+ my @ALLOWED_METHODS = qw( foo bar );
 
  sub foo {
      # ...
@@ -761,8 +759,6 @@ L<Apache2::Controller::Uploads>
 
 L<Apache2::Controller::Session>
 
-L<Apache2::Controller::DBI::Connector>
-
 L<Apache2::Controller::Auth::OpenID>
 
 L<Apache2::Controller::X>
@@ -792,3 +788,18 @@ it under the same terms as Perl itself.
 =cut
 
 1;
+
+__END__ 
+=for comment
+
+This stuff is for somewhere else, needs to be split from Dolph stuff
+
+sub save_session {
+    my ($self) = @_;
+    my ($dbh, $sid, $session) = @{$self}{qw( dbh sid session )};
+    $dbh->do(
+        q{ UPDATE session SET yaml = ? WHERE sid = ? },
+        undef, Dump($session), $sid
+    );
+    return;
+}
