@@ -1,17 +1,7 @@
-package Apache2::Controller::Funk;
-
 =head1 NAME
 
 Apache2::Controller::Funk
 
-=head1 VERSION
-
-Version 0.110.000 - BETA TESTING (ALPHA?)
-
-=cut
-
-our $VERSION = version->new('0.110.000');
-
 =head1 SYNOPSIS
 
  $bool = controller_allows_method($class, $method); 
@@ -27,8 +17,10 @@ queried futher using 'exists', which is very fast.
 
 =cut
 
+package Apache2::Controller::Funk;
+
 use strict;
-use warnings FATAL => 'all';
+use warnings FATAL => 'all', NONFATAL => 'redefine';
 use English '-no_match_vars';
 
 use base 'Exporter';
@@ -53,12 +45,11 @@ Readonly my $ACCESS_LOG_REASON_LENGTH => 60;
 
 =head1 IMPORTABLE FUNCTIONS
 
-=head2 controller_allows_method
+=head2 controller_allows_method( )
 
  $bool = controller_allows_method($class, $method); # controller_allows_method()
 
-Ask if method name is returned by C<< allowed_methods() >>
-in the given controller package.
+Ask if method name is in @ALLOWED_METHODS in the given controller package.
 
 Only two 'exists' calls are required for each query after caching the 
 first result for this child.
@@ -90,10 +81,13 @@ sub controller_allows_method {
         Apache2::Controller::X->throw("$class is not an Apache2::Controller")
             unless $isa_a2c;
 
-        Apache2::Controller::X->throw("$class knows no allowed_methods()")
-            unless $class->can('allowed_methods');
-
-        my @allowed_methods = $class->allowed_methods();
+        my @allowed_methods = ( );
+        my $stmt = '@allowed_methods = @'.$class.'::ALLOWED_METHODS;';
+        DEBUG("stmt: '$stmt'");
+        eval $stmt;
+        Apache2::Controller::X->throw(
+            "$class does not have \@ALLOWED_METHODS: $EVAL_ERROR"
+        ) if $EVAL_ERROR;
 
         DEBUG("allowed_methods: (@allowed_methods)");
         $allowed_methods{$class} = { map {($_=>1)} @allowed_methods };
@@ -102,12 +96,12 @@ sub controller_allows_method {
     return exists $allowed_methods{$class}{$method};
 }
 
-=head2 check_allowed_method
+=head2 check_allowed_method( )
 
  check_allowed_method($method, $class); # check_allowed_method()
 
 Throw a NOT_FOUND exception if the method is not an allowed method
-in the C<< allowed_methods() >> list in the controller package.
+in the @ALLOWED_METHODS array in the controller package.
 
 =cut
 
@@ -121,7 +115,7 @@ sub check_allowed_method {
         DEBUG("Method $method not allowed in $class.");
         Apache2::Controller::X->throw(
             message     => "Method $method not allowed from $class.",
-            status => Apache2::Const::NOT_FOUND,
+            http_status => Apache2::Const::NOT_FOUND,
         );
     }
     return;
