@@ -5,14 +5,6 @@ package Apache2::Controller::NonResponseBase;
 Apache2::Controller::NonResponseBase - internal base class for 
 non-response handlers in Apache2::Controller framework
 
-=head1 VERSION
-
-Version 0.110.000 - BETA TESTING (ALPHA?)
-
-=cut
-
-our $VERSION = version->new('0.110.000');
-
 =head1 SYNOPSIS
 
 This is an INTERNAL base class and you don't need to use it.
@@ -47,7 +39,7 @@ processing the chain of handlers... we'll see.
 =cut
 
 use strict;
-use warnings FATAL => 'all';
+use warnings;
 use English '-no_match_vars';
 
 use Log::Log4perl qw(:easy);
@@ -55,7 +47,6 @@ use YAML::Syck;
 
 use Apache2::RequestRec ();
 use Apache2::RequestUtil ();
-use Apache2::Log;
 use Apache2::Const -compile => qw( :common :http :methods );
 
 use Apache2::Controller::X;
@@ -74,7 +65,7 @@ sub handler : method {
 
     DEBUG("begin $class ->handler()");
 
-    my ($handler, $status, $X) = ( );
+    my ($handler, $http_status, $X) = ( );
 
     eval { 
         $handler = $class->new($r);
@@ -82,45 +73,44 @@ sub handler : method {
     };
     if ($X = Exception::Class->caught('Apache2::Controller::X')) {
         if ($X->isa('Apache2::Controller::X::Redirect')) {
-            $status = Apache2::Const::REDIRECT;
+            $http_status = Apache2::Const::REDIRECT;
             $r->err_headers_out->add(Location => "$X");
             DEBUG("redirecting to $X");
         }
         else {
-            $status = $X->status || Apache2::Const::SERVER_ERROR;
-            DEBUG("Caught an Apache2::Controller::X: $status");
+            $http_status = $X->http_status || Apache2::Const::SERVER_ERROR;
+            DEBUG("Caught an Apache2::Controller::X: $http_status");
             DEBUG(ref($X).": $X\n".($X->dump ? Dump($X->dump) : '').$X->trace());
         }
     }
     elsif ($X = $EVAL_ERROR) {
         DEBUG("Caught an unknown error: $X");
-        $status = Apache2::Const::SERVER_ERROR;
+        $http_status = Apache2::Const::SERVER_ERROR;
     }
 
-    if ($status) {
-        DEBUG("Setting http-status to '$status'");
-        $r->status($status);
+    if ($http_status) {
+        DEBUG("Setting http-status to '$http_status'");
+        $r->status($http_status);
     }
 
-    if ($status && $status >= Apache2::Const::HTTP_BAD_REQUEST) {
+    if ($http_status && $http_status >= Apache2::Const::HTTP_BAD_REQUEST) {
         DEBUG("logging bad request");
-        eval { log_bad_request_reason($r, $X); };
-        if (my $X = Exception::Class->caught('Apache2::Controller::X')) {
-            FATAL("Bad error logging bad request! '$X'\n".$X->trace);
-        }
-        elsif ($EVAL_ERROR) {
-            FATAL("Weird error logging bad request! '$EVAL_ERROR'");
-        }
+        log_bad_request_reason($r, $X);
+        return Apache2::Const::DONE;  # stop processing further handlers
+    }
+    elsif ($http_status && $http_status == Apache2::Const::REDIRECT) {
+        return Apache2::Const::DONE;
+    }
+    else {
+        DEBUG("returning OK");
+        return Apache2::Const::OK;
     }
-    DEBUG("returning OK");
-    return Apache2::Const::OK;
 }
 
 =head2 new
 
 C<new()> creates an object of the child class and assigns the
-C<< Apache2::RequestRec >> object to 
-C<< $self->{r} >>.
+C<<Apache2::RequestRec>> object to C<<$self->{r}>>.
 
 If the parent class defines a method C<init()>, this will
 be called at the end of object creation.
@@ -128,13 +118,10 @@ be called at the end of object creation.
 Unlike L<Apache2::Controller>, the handler object of other handlers
 that use this package as a base do not create, delegate to and subclass
 the L<Apache2::Request> object.  They just keep the original 
-L<Apache2::RequestRec> object in 
-C<< $self->{r} >>.
+L<Apache2::RequestRec> object in C<<$self->{r}>>.
 
 =cut
 
-my %can_init;
-
 sub new {
     my ($class, $r) = @_;
 
@@ -146,8 +133,7 @@ sub new {
     };
     bless $self, $class;
 
-    $can_init{$class} = $self->can('init') if !exists $can_init{$class};
-    $self->init() if $can_init{$class};
+    $self->init() if $self->can('init');
 
     return $self;
 }
