@@ -4,37 +4,24 @@ package Apache2::Controller::Render::Template;
 
 Apache2::Controller::Render::Template - A2C render() with Template Toolkit
 
-=head1 VERSION
-
-Version 0.110.000 - BETA TESTING (ALPHA?)
-
-=cut
-
-our $VERSION = version->new('0.110.000');
-
 =head1 SYNOPSIS
 
  # apache2 config file
 
- PerlLoadModule Apache2::Controller::Directives
- PerlLoadModule Apache2::Controller::DBI::Connector
+ PerlModule Apache2::Controller::Directives
 
  # location of templates - must be defined
- A2C_Render_Template_Path  /var/myapp/templates /var/myapp/template_components
+ A2CRenderTemplateDir           /var/myapp/templates
 
+ # looks for templates in /var/myapp/templates/foo/
  <Location /foo>
     SetHandler modperl
-    PerlInitHandler             MyApp::Dispatch::Foo
-
-    # set directives A2C_DBI_DSN, etc. 
-    PerlHeaderParserHandler     Apache2::Controller::DBI::Connector
+    PerlInitHandler MyApp::Dispatch::Foo
  </Location>
 
-See L<Apache2::Controller::Dispatch> for A2C Dispatch implementations.
+ # see L<Apache2::Controller::Dispatch> for A2C Dispatch inheritance.
 
-See L<Apache2::Controller::Directives> and L<Apache2::Controller::DBI::Connector>.
-
- package MyApp::C::Bar;  # let's assume this controller was dispatched
+ package MyApp::C::Foo;  # let's assume this controller was dispatched
 
  use strict;
  use warnings;
@@ -42,17 +29,23 @@ See L<Apache2::Controller::Directives> and L<Apache2::Controller::DBI::Connector
  use base qw(
     Apache2::Controller
     Apache2::Controller::Render::Template
+    MyApp::Model::Methods
+    MyApp::Security
  );
 
  use Apache2::Const -complie => qw( OK );
 
- sub allowed_methods {qw( default )}
+ my @ALLOWED_METHODS = qw( default );
+
+ # suppose MyApp::Model::Methods connects dbh in startup sequence
+ # and suppose MyApp::Security implements my_detaint_path_args()
+ # to call a branch function for 'name' that detaints [ last, first ]
 
  sub default {
-    my ($self, @first, @last) = @_;
+    my ($self) = @_;
     my @path_args = $self->my_detaint_path_args('name'); # from $self->{path_args}
 
-    $self->{stash}{creditcards} = $self->pnotes->{dbh}->fetchall_arrayref(
+    $self->{stash}{creditcards} = $self->{dbh}->fetchall_arrayref(
         q{  SELECT ccnum, exp, addr1, zip, cac 
             FROM customer_credit_cards 
             WHERE lname = ? AND fname = ?
@@ -84,42 +77,6 @@ See L<Apache2::Controller::Directives> and L<Apache2::Controller::DBI::Connector
 
 This module provides a nice rendering mechanism for Apache2::Controller.
 
-=head1 TEMPLATE OPTIONS
-
-You can specify options for L<Template> in one of two ways:
-
-=head2 DIRECTIVES
-
-By using 
-L<Apache2::Controller::Directives/A2C_Render_Template_Opts>:
-
- <Location '/foo'>
-     A2C_Render_Template_Opts INTERPOLATE 1
-     A2C_Render_Template_Opts PRE_PROCESS header
-     A2C_Render_Template_Opts POST_CHOMP  1
- </Location>
-
-=head2 METHOD template_options()
-
-Or by implementing C<<template_options>> in your controller:
-
- package MyApp::Controller;
- use base qw( Apache2::Controller Apache2::Controller::Render::Template );
- sub allowed_methods {qw( default )}
-
- sub template_options {
-     my ($self) = @_;
-     return {
-         INTERPOLATE => 1,
-         PRE_PROCESS => 'header',
-         POST_CHOMP  =. 1,
-     };
- }
-
- sub default { 
-    # ...
- }
-
 =head1 STASH FUNCTIONS
 
 Several subroutine references are automatically included
@@ -265,7 +222,7 @@ sub render_fast {
 If your template directory contains a subdirectory named 'error', 
 then when the controller throws an exception, the exception object will
 be passed to a selected error template as 'X' in the stash.  It also
-sets status (number) and status_line 
+sets http_status (number) and status_line 
 (from HTTP::Status::status_message() or from the values 
 set in the L<Apache2::Controller::X> exception).
 
@@ -290,7 +247,7 @@ error() remembers across requests whether you do or don't have
 error templates for certain messages in the appropriate template directory,
 so it will be faster the second time around if you use error/default.html.
 
-For a reference list of status and messages, see Apache2::Controller.
+For a reference list of http_status and messages, see Apache2::Controller.
 
 Since render_fast() is incompatible if a template rendering error 
 occurs, render_fast() turns off the use of error() and relies on 
@@ -304,27 +261,27 @@ my %error_templates = ( );
 sub error {
     my ($self, $X) = @_;
 
-    my ($status, $status_line);
+    my ($http_status, $status_line);
 
     DEBUG("original error: '$X'");
 
     if (ref($X) && $X->isa('Apache2::Controller::X')) {
-        $status = $X->status;
+        $http_status = $X->http_status;
         $status_line = $X->status_line;
-        DEBUG("got status from \$X: ".($status || '[none]'));
+        DEBUG("got status from \$X: ".($http_status || '[none]'));
     }
-    $status ||= Apache2::Const::SERVER_ERROR;
-    $status_line ||= status_message($status);
+    $http_status ||= Apache2::Const::SERVER_ERROR;
+    $status_line ||= status_message($http_status);
 
-    my $status_file = $status;
+    my $status_file = $http_status;
     DEBUG("status msg for $status_file: '$status_line'");
 
     $self->{stash}{X} = $X;
     $self->{stash}{status_line} = $status_line;
-    $self->{stash}{status} = $status;
+    $self->{stash}{http_status} = $http_status;
 
-    my $template_dir = $self->get_directive('A2C_Render_Template_Path')
-        || Apache2::Controller::X->throw('A2C_Render_Template_Path not defined.');
+    my $template_dir = $self->get_directive('A2CRenderTemplateDir')
+        || Apache2::Controller::X->throw('A2CRenderTemplateDir not defined.');
     if (exists $error_templates{$template_dir}{$status_file}) {
 
         my $template = $error_templates{$template_dir}{$status_file};
@@ -363,14 +320,14 @@ sub error {
             if ($EVAL_ERROR) {
                 $try_errors{$self->{template}} = "$EVAL_ERROR";
                 $error_templates{$template_dir}{$status_file} = undef;
-                if (ref $X && $X->isa('Apache2::Controller::X')) {
+                if ($X->isa('Apache2::Controller::X')) {
                     $X->rethrow();
                 }
                 else {
                     my $dump = { tries => \%try_errors, reftype => ref $X };
                     Apache2::Controller::X->throw(
-                        message     => "$X",
-                        status => $status,
+                        message     => "GOMBOR $X",
+                        http_status => $http_status,
                         status_line => $status_line,
                         'dump'      => $dump,
                     );
@@ -393,12 +350,12 @@ To override the auto-select template, just set $self->{template}
 before you call C<<render()>>.
 
 It looks for templates in a computed directory.  The directory where it
-looks will always be the directory set with the A2C_Render_Template_Path 
+looks will always be the directory set with the A2CRenderTemplateDir 
 directive in the config file, appended with the current request location,
 i.e. the directory of the Location directive in the config file, 
 appended with relative_uri, appended with method name and '.html'.
 
- A2C_Render_Template_Path + location + relative_uri + method.html
+ A2CRenderTemplateDir + location + relative_uri + method.html
 
 For example, the sequence in SYNOPSIS above renders the file 
 C</var/myapp/templates/foo/default.html> .
@@ -442,31 +399,16 @@ sub detect_template {
         return $self->{template};
     }
 
-    my $loc = $self->location();
-
-    # trim leading and trailing /'s
-    $loc = '' if $loc eq '/';
-    if ($loc) {
-        $loc = substr($loc, 1) if length $loc > 1 && substr($loc, 0, 1) eq '/';
-        substr($loc, -1, 1, '') if substr($loc, -1) eq '/';
-    }
-
     (my $rel_uri = $self->notes->{relative_uri}) =~ s{ \A / }{}mxs;
     Apache2::Controller::X->throw('notes->{relative_uri} not set')
         if !defined $rel_uri;
 
     my $file = "$self->{method}.html";
 
-    DEBUG(sub{Dump({
-        loc         => $loc,
-        rel_uri     => $rel_uri,
-        file        => $file,
-    })});
-
     my $template 
         = $rel_uri
-        ? File::Spec->catfile( $loc, $rel_uri, $file )
-        : File::Spec->catfile( $loc, $file );
+        ? File::Spec->catfile( $rel_uri, $file )
+        : $file;
 
     Apache2::Controller::X->throw("bad template path $_")
         if $template =~ m{ \.\. / }mxs;
@@ -477,50 +419,65 @@ sub detect_template {
     return $template;
 }
 
-=head2 get_tt_obj
+=head2 detect_template_include_dir
 
+Like C<<detect_template()>>, this detects the appropriate include
+directory for the template toolkit object, sets it as 
+C<<$self->{template_include_dir}>> and returns it.
+
+=cut
+
+sub detect_template_include_dir {
+    my ($self) = @_;
+
+    return $self->{template_include_dir} if $self->{template_include_dir};
+
+    my $template_dir = $self->get_directive('A2CRenderTemplateDir')
+        || Apache2::Controller::X->throw("A2CRenderTemplateDir not defined");
+
+    (my $loc = $self->location()) =~ s{ \A / }{}mxs;
+    DEBUG("so far have '$template_dir' / '$loc'");
+    my $uri = $self->uri();
+
+    DEBUG(sub{Dump({
+        loc     => $loc,
+        uri     => $uri,
+        template_dir => $template_dir,
+    })});
+
+    my $dir = $self->{template_include_dir} = File::Spec->catfile(
+        $template_dir,
+        $loc,
+    );
+
+    return $dir;
+}
+
+=head2 get_tt_obj
+
 Get the L<Template> object set up with the appropriate include directory
-set from the directive C<<A2C_Render_Template_Path>>.
-
-Directive C<A2C_Render_Template_Opts> sets default C<new()>
-options for L<Template>.  
+from C<<detect_template_include_dir()>>.
 
 =cut
 
-my %implements_template_opts;
 my %tts = ();
 sub get_tt_obj {
     my ($self) = @_;
+    my $include_dir = $self->detect_template_include_dir();
 
-    my $include_path = $self->get_directive('A2C_Render_Template_Path')
-        || Apache2::Controller::X->throw("A2C_Render_Template_Path not defined");
+    return $tts{$include_dir} if exists $tts{$include_dir};
 
-    my $class = $self->{class};
-    $implements_template_opts{$class} = $self->can('template_opts')
-        if !exists $implements_template_opts{$class};
+    DEBUG("using include_dir '$include_dir' to set up a new TT object");
 
-    my %opts 
-        = $implements_template_opts{$class}
-        ? %{ $self->template_opts() }
-        : %{ $self->get_directive('A2C_Render_Template_Opts') || { } };
+    my $tt = Template->new({
+        INCLUDE_PATH    => $include_dir,
+        INTERPOLATE     => 1,
+        ABSOLUTE        => 0,
+        RELATIVE        => 1,
+    # DEBUG           => 'all',
+    }) || die $Template::ERROR;
 
-    $opts{INCLUDE_PATH} 
-        = !$opts{INCLUDE_PATH}    ? $include_path
-        : ref $opts{INCLUDE_PATH} ? [@{$opts{INCLUDE_PATH}}, @{$include_path}]
-        :                           [$opts{INCLUDE_PATH}, @{$include_path}];
-
-    my $opts_key = Dump(\%opts);
-    DEBUG("using opts for new TT on $include_path:\n$opts_key");
-
-    return $tts{$opts_key} if exists $tts{$opts_key};
-
-    my $tt;
-    
-    eval { $tt = Template->new(\%opts) || die $Template::ERROR."\n" };
-    Apache2::Controller::X->throw("No TT for $include_path: $EVAL_ERROR") 
-        if $EVAL_ERROR;
-
-    $tts{$opts_key} = $tt;
+    $tts{$include_dir} = $tt;
 
     return $tt;
 }
