@@ -1,17 +1,12 @@
 package Apache2::Controller::X;
 
+use warnings;
+use strict;
+
 =head1 NAME
 
 Apache2::Controller::X - Exception::Class hierarchy for Apache2::Controller
 
-=head1 VERSION
-
-Version 0.110.000 - BETA TESTING (ALPHA?)
-
-=cut
-
-our $VERSION = version->new('0.110.000');
-
 =head1 SYNOPSIS
 
  package MyApp::C::Foo;
@@ -43,7 +38,7 @@ our $VERSION = version->new('0.110.000');
  sub page_controller_method {
      MyApp::X->throw(
           message => q{ You're not supposed to be here. },
-          status => Apache2::Const::FORBIDDEN,
+          http_status => Apache2::Const::FORBIDDEN,
           dump => {
             this    => q{structure will get YAML::Syck::Dump'd},
             that    => [qw( to the error log )],
@@ -73,24 +68,24 @@ field, which is displayed when the object is referred to in string context.
      warn $X->trace;
  }
 
-=head2 status
+=head2 http_status
 
 This can be set to an L<Apache2::Const/:http> constant, which
-will then be set as the status for the request.
+will then be set as the http_status for the request.
 
  Apache2::Controller::X->throw(
      message => "oh no!",
-     status => Apache2::Const::HTTP_INTERNAL_SERVER_ERROR,
+     http_status => Apache2::Const::HTTP_INTERNAL_SERVER_ERROR,
  );
 
 =head2 status_line
 
-Combined with status, when intercepted by L<Apache2::Controller/handler>
+Combined with http_status, when intercepted by L<Apache2::Controller/handler>
 this sets a custom message with L<Apache2::RequestRec/status_line>.
 
  Apache2::Controller::X->throw(
      message => "Warp injection coil failure in unit 3-A-73",
-     status => Apache2::Const::HTTP_INTERNAL_SERVER_ERROR,
+     http_status => Apache2::Const::HTTP_INTERNAL_SERVER_ERROR,
      status_line => "Turbulence ahead. Please buckle your safety belts.",
  );
 
@@ -125,12 +120,9 @@ $self->internal_redirect() inherited from Apache2::SubRequest.
 
 =cut
 
-use warnings FATAL => 'all';
-use strict;
-
 use Exception::Class (
     'Apache2::Controller::X'           => { 
-        fields  => [qw( message dump status status_line )],
+        fields  => [qw( message dump http_status status_line )],
     },
     'Apache2::Controller::X::Redirect' => { 
         isa => 'Apache2::Controller::X',
