@@ -3,21 +3,16 @@
 #warn "@INC";
 BEGIN {
 
-use strict; 
-use warnings;
 use English '-no_match_vars';
-use YAML::Syck;
-
-# warn "INC: (@INC)\n";
 
 use Log::Log4perl qw(:easy);
-my $loginit = q{
+my $logconf = q{
 log4perl.rootLogger=DEBUG, Screen
 log4perl.appender.Screen=Log::Log4perl::Appender::Screen
 log4perl.appender.Screen.layout=PatternLayout
 log4perl.appender.Screen.layout.ConversionPattern=----------------------------------%n%p %M() %L:%n%m%n
 };
-Log::Log4perl->init(\$loginit);
+Log::Log4perl->init(\$logconf);
 
 use File::Spec;
 my $tmp = File::Spec->tmpdir();
@@ -25,7 +20,7 @@ my $tmp = File::Spec->tmpdir();
 my $dir = File::Spec->catfile($tmp, 'A2Ctest');
 
 do {
-  # DEBUG("Creating temp directory $_");
+    DEBUG("Creating temp directory $_");
     mkdir || die "Cannot create $_: $OS_ERROR\n";
 } for grep !-d, 
     $dir, 
