@@ -14,16 +14,19 @@ use Apache2::Const -compile => qw(HTTP_OK);
 use Log::Log4perl qw(:easy);
 use YAML::Syck;
 
-sub allowed_methods {qw( set read )}
+Readonly our @ALLOWED_METHODS => qw( set read );
+our %testdata = (
+    foo     => {
+        boz     => [qw( noz schnoz )]
+    },
+    bar     => 'biz',
+);
 
 sub set {
     my ($self) = @_;
     DEBUG('setting session data');
     $self->content_type('text/plain');
-    my ($data) = $self->param('data') =~ m{ \A (.*?) \z }mxs;
-    die "no data param!\n" if !$data;
-    DEBUG("received param data:\n$data|||\n");
-    $self->{session}{testdata} = Load($data);
+    $self->{session}{testdata}{$_} = $testdata{$_} for keys %testdata;
     DEBUG(sub { "session data is now:\n".Dump($self->{session}) });
     $self->print("Set session data.\n");
     return Apache2::Const::HTTP_OK;
